cdef cypclass CrazyOperations:
    int a
    double b

    CrazyOperations __iadd__(self, CrazyOperations other):
        self.a += other.a

    CrazyOperations __add__(self, CrazyOperations other):
        return CrazyOperations(self.a, self.b + other.b)

    CrazyOperations __isub__(self, CrazyOperations other):
        self.b -= other.b

    __init__(self, int a = 0, double b = 4.2):
        self.a = a
        self.b = b

cdef cypclass SuperCrazyOperations(CrazyOperations):

    __init__(self, int a = 0, double b = 4.2):
        CrazyOperations.__init__(self, a, b)

    SuperCrazyOperations __iadd__(self, CrazyOperations other):
        CrazyOperations.__iadd__(self, other)

    SuperCrazyOperations __iadd__(self, SuperCrazyOperations other):
        # This is one (possibly weird) way to chain to base class' __iadd__, done here for the example
        casted_self = <CrazyOperations> self
        # As this is casted, we will call base __iadd__ with the += below
        casted_self += <CrazyOperations> other
        self.b = 0
        # At the end, we added both a attributes, and set out b to 0.0

    SuperCrazyOperations __isub__(self, CrazyOperations other):
        CrazyOperations.__isub__(self, other)

    SuperCrazyOperations __isub__(self, SuperCrazyOperations other):
        CrazyOperations.__isub__(self, other)
        self.a = 4


def test():
    o1 = SuperCrazyOperations(1, 2)
    o2 = CrazyOperations(42, 4.2)
    o3 = SuperCrazyOperations(-3, 2.1)

    # This should print:
    # 1 2.0
    print(o1.a, o1.b)

    # This should print:
    # 42 4.2
    print(o2.a, o2.b)

    # This should print:
    # -3 2.1
    print(o3.a, o3.b)

    # We're calling __add__ so we just add the b attributes
    tmp1 = o1 + o2
    # This should print:
    # 1 6.2
    print(tmp1.a, tmp1.b)

    # CrazyOperations' __iadd__ is called: adding the a attributes 
    o2 += o1
    # This should print:
    # 43 4.2
    print(o2.a, o2.b)

    # SuperCrazyOperations' __iadd__ is called: adding the a attributes AND setting own (o1) b to 0.0 
    o1 += o3
    # This should print:
    # -2 0
    print(o1.a, o1.b)

    # We're calling CrazyOperations' __isub__, so we're substracting b attributes
    o1 -= o2
    # This should print:
    # -2 -4.2
    print(o1.a, o1.b)

    # We're calling SuperCrazyOperations' __isub__, so we're substracting b attributes AND setting own (o1) a to 4
    o1 -= o3
    # This should print:
    # 4 -6.3
    print(o1.a, o1.b)